﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.SimulationCore {
  [StorableClass]
  public sealed class CompositeEventQueue<TModel> : Item where TModel : IModel {
    [Storable]
    private List<IEventQueue<TModel>> queues;

    public bool Empty {
      get { return queues.All(x => x.Empty); }
    }

    public int Count {
      get { return queues.Sum(x => x.Count); }
    }

    public IEvent<TModel> Peek() {
      IEvent<TModel> nextEvent = null;
      foreach (var queue in queues) {
        if (!queue.Empty && (nextEvent == null || queue.Peek().Time < nextEvent.Time)) {
          nextEvent = queue.Peek();
        }
      }
      return nextEvent;
    }

    public IEvent<TModel> Pop() {
      IEvent<TModel> nextEvent = null;
      IEventQueue<TModel> minEventQueue = null;
      foreach (var queue in queues) {
        if (!queue.Empty && (nextEvent == null || queue.Peek().Time < nextEvent.Time)) {
          nextEvent = queue.Peek();
          minEventQueue = queue;
        }
      }
      return minEventQueue.Pop();
    }

    public void Clear() {
      foreach (var queue in queues) {
        queue.Clear();
      }
    }

    [StorableConstructor]
    private CompositeEventQueue(bool deserializing) : base(deserializing) { }
    private CompositeEventQueue(CompositeEventQueue<TModel> original, Cloner cloner)
      : base(original, cloner) {
      this.queues = original.queues.Select(cloner.Clone).ToList();
    }
    public CompositeEventQueue(IEnumerable<IEventQueue<TModel>> queues) {
      this.queues = new List<IEventQueue<TModel>>(queues);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CompositeEventQueue<TModel>(this, cloner);
    }
  }
}
